/*
Module for export project files
*/var fs, util;
var __indexOf = Array.prototype.indexOf || function(item) {
  for (var i = 0, l = this.length; i < l; i++) {
    if (this[i] === item) return i;
  }
  return -1;
};
fs = require("fs");
util = require("./util");
/*
Export tracker to txt file

@param {Object} tracker Tracker object
@param {String} filename Path to file
@api public
*/
exports.toTxt = function(tracker, filename) {
  var c, k, out, t, _i, _len, _ref, _ref2;
  out = "" + tracker.name + "\n";
  _ref = tracker.tickets;
  for (k in _ref) {
    t = _ref[k];
    out += "created: " + t.created;
    if (t.created !== t.modified) {
      out += "\tmodified: " + t.modified;
    }
    out += "\n" + t.author.user + " <" + t.author.email + ">\n" + t.text + "\n";
    if (0 < t.comments.length) {
      out += "Comments :\n";
    }
    _ref2 = t.comments;
    for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
      c = _ref2[_i];
      out += "" + c.author + ":\n" + c.comment + "\n";
    }
    out += "\n";
  }
  return fs.writeFileSync(filename, out);
};
/*
Export tracker to org-mode file

@param {Object} tracker Tracker object
@param {String} filename Path to file
@api public
*/
exports.toOrg = function(tracker, filename) {
  var c, firstLine, out, restLines, t, tags, text, _i, _j, _len, _len2, _ref, _ref2, _ref3;
  out = "#+STARTUP: hidestars\n#+STARTUP: align\n#+SEQ_TODO:";
  tracker.states.initial.forEach(function(state) {
    return out += "" + (state.toUpperCase()) + " ";
  });
  out += "| ";
  tracker.states.final.forEach(function(state) {
    return out += "" + (state.toUpperCase()) + " ";
  });
  out += "\n#+AUTHOR: " + (tracker.config.get('user')) + "\n#+EMAIL: " + (tracker.config.get('email')) + "\n";
  out += "\n\n* " + (tracker.name || tracker.config.get('user') + '\'s project') + "[/]\n";
  _ref = tracker._sortTickets();
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    t = _ref[_i];
    _ref2 = util.searchAndReplaceTags(t.text, function(tag) {
      return tag;
    }), text = _ref2[0], tags = _ref2[1];
    text = text.replace('@' + t.state, '').replace(/\n/g, '\n   ');
    text = text.split("\n");
    firstLine = text[0];
    restLines = text.slice(1);
    tags = tags.join(":").replace(/[\-\.\,]/g, "");
    if (tags) {
      tags = ":" + tags + ":";
    }
    out += ("** " + (t.state.toUpperCase()) + " ") + ("" + (util.makeLineLonger(firstLine)) + " " + tags + "\n");
    out += "" + (restLines.join('\n'));
    if (0 < t.comments.length) {
      out += "\n*** Comments\n";
      _ref3 = t.comments;
      for (_j = 0, _len2 = _ref3.length; _j < _len2; _j++) {
        c = _ref3[_j];
        out += "    - " + (c.comment.replace(/\n/g, '\n     ')) + "\n";
      }
    }
    out += "\n";
  }
  return fs.writeFileSync(filename, out);
};
/*
Export tracker to html file

@param {Object} tracker Tracker object
@param {String} filename Path to file
@api public
*/
exports.toHtml = function(tracker, filename) {
  var out, styles, t, tags, text, _i, _len, _ref, _ref2, _ref3;
  out = '<html><head><meta charset="utf-8"><title>' + ("" + (tracker.name || tracker.config.get('user') + '\'s project' & mdash(config.get('user')))) + '</title><body><ul style="list-style:none;">\n';
  _ref = tracker._sortTickets();
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    t = _ref[_i];
    _ref2 = util.searchAndReplaceTags(t.text, function(tag) {
      return "<a href=\"#\">" + tag + "</a>";
    }), text = _ref2[0], tags = _ref2[1];
    text = text.replace(/\n/g, "<br />");
    if (_ref3 = util.getState(t.text, tracker.config), __indexOf.call(tracker.states.final, _ref3) >= 0) {
      styles = 'style="color:grey"';
      text = "<s>" + text + "</s>";
    } else {
      styles = "";
    }
    out += "<li " + styles + ">" + text + " </li>\n";
  }
  out += "</ul>\n</body></html>\n";
  return fs.writeFileSync(filename, out);
};