/*
Sources parser module
*/var extractCommentsText, fs, path, removeStringsFromLine, sourceOptions, util;
fs = require("fs");
path = require("path");
util = require("./util");
/*
Options for various programming languages
*/
sourceOptions = {
  js: {
    oneLineComment: "//",
    multilineCommentStart: "/*",
    multilineCommentEnd: "*/",
    escapeStringQuotesRe: [/\\\'/g, /\\\"/g],
    stringRe: [new RegExp("'[^']+'", "g"), new RegExp('"[^"]+"', "g")]
  },
  coffee: {
    oneLineComment: "#",
    multilineCommentStart: "###",
    multilineCommentEnd: "###",
    escapeStringQuotesRe: [/\\\'/g, /\\\"/g],
    stringRe: [new RegExp("'[^']+'", "g"), new RegExp('"[^"]+"', "g")]
  },
  py: {
    oneLineComment: "#",
    multilineCommentStart: null,
    multilineCommentEnd: null,
    escapeStringQuotesRe: [/\\\'/g, /\\\"/g],
    stringRe: [new RegExp("'[^']+'", "g"), new RegExp('"[^"]+"', "g")]
  },
  rb: {
    oneLineComment: "#",
    multilineCommentStart: "=begin",
    multilineCommentEnd: "=end",
    escapeStringQuotesRe: [/\\\'/g, /\\\"/g],
    stringRe: [new RegExp("'[^']+'", "g"), new RegExp('"[^"]+"', "g")]
  }
};
/*
Remove strings from line

@param {String} line Source string
@param {Object} opts Language-specific options
@return {String} result Line without quoted strings
@api private
*/
removeStringsFromLine = function(line, opts) {
  var re, _i, _j, _len, _len2, _ref, _ref2;
  _ref = opts.escapeStringQuotesRe;
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    re = _ref[_i];
    line = line.replace(re, "");
  }
  _ref2 = opts.stringRe;
  for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
    re = _ref2[_j];
    line = line.replace(re, "");
  }
  return line;
};
/*
Extract comments from line

@param {String} line Source string
@param {Object} opts Language-specific options
@param {Boolean} mlCommentOpen Flag, that set if multiline comment open in prevoius line
@return {Array} result Return array, that consist of comment string and new mlCommentOpen flag
@api public
*/
extractCommentsText = function(line, opts, mlCommentOpen) {
  var mlEnd, mlStart, mlcLength, offset, olComment;
  mlStart = line.indexOf(opts.multilineCommentStart);
  offset = opts.multilineCommentStart === opts.multilineCommentEnd ? 0 : mlStart + 1;
  mlEnd = line.indexOf(opts.multilineCommentEnd, offset);
  olComment = line.indexOf(opts.oneLineComment);
  if (mlCommentOpen) {
    if (0 <= mlEnd) {
      return [line.substring(0, line.indexOf(opts.multilineCommentEnd)), false];
    } else {
      return [line, true];
    }
  } else {
    mlcLength = opts.multilineCommentStart ? opts.multilineCommentStart.length : 0;
    if (0 <= mlStart) {
      if (mlStart < mlEnd) {
        return [line.substring(mlStart + mlcLength, mlEnd), false];
      } else {
        return [line.substring(mlStart + mlcLength), true];
      }
    } else if (0 <= olComment) {
      return [line.substring(olComment + opts.oneLineComment.length), false];
    } else {
      return ["", false];
    }
  }
};
/*
Add tickets from source file.
To all tickets add tags: +auto and +(filename.ext)

@param {String} file File path
@param {Array} tags State tags
@param {Object} tracker Tracker object
@param {Object} config Config object
@api public
*/
exports.addTickets = function(file, tags, tracker, config) {
  var curLines, linesMax, opts, ticketText;
  opts = sourceOptions[path.extname(file).substring(1)];
  if (opts) {
    linesMax = parseInt(config.get("maxlinesAfterState"));
    linesMax || (linesMax = 3);
    curLines = 1;
    ticketText = "";
    return fs.readFile(file, function(err, data) {
      var comment, i, l, line, mlCommentOpen, t, _i, _len, _ref, _ref2, _results;
      if (null === err) {
        i = 0;
        mlCommentOpen = false;
        _ref = data.toString().split("\n");
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          line = _ref[_i];
          l = removeStringsFromLine(line, opts);
          _ref2 = extractCommentsText(l, opts, mlCommentOpen), comment = _ref2[0], mlCommentOpen = _ref2[1];
          i++;
          _results.push((function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = tags.length; _i < _len; _i++) {
              t = tags[_i];
              _results.push(0 <= comment.toLowerCase().indexOf(t) ? (ticketText ? (tracker.addUniqueTicket(ticketText + "\n..."), ticketText = "") : ticketText = comment.trim().replace(t, util.statePrefix + t) + " +auto +" + util.extractPartialPath(file) + "\n" + line, curLines = 1) : ticketText ? curLines < linesMax ? (ticketText += "\n" + line, curLines++) : (tracker.addUniqueTicket(ticketText + "\n..."), ticketText = "", curLines = 1) : void 0);
            }
            return _results;
          })());
        }
        return _results;
      }
    });
  }
};