var Tracker, exp, parser, showHelp, sys, usage, util;
require("colors");
util = require("./util");
Tracker = require("./tracker").Tracker;
sys = require("sys");
parser = require("./source-parser");
exp = require("./export");
usage = 'Usage:\n\nstrack [COMMAND] args\n\nid field in commands may be replaced by shortuts ^0-^9 and ^a-^z\n\nstrack commands and aliases:\n  add, a\tAdd new ticket/task\n  config, cf\tWork with config options\n  comment, c\tComment ticket/task\n  done, d\tChange ticket/task state\n  fs    \tSearch tags in source\n  edit, e\tEdit ticket title\n  help, h\tHelp on commands\n  info, i\tShow info on ticket/task\n  log, l\tShow tracker log.\n        \tSearch by tags, states and regular words\n  remove, rm\tRemove ticket/task\n  states, st\tChange states for project\n  touch, t\tChange task access time';
showHelp = function() {
  var v;
  v = process.argv[3];
  switch (v) {
    case "add":
    case "a":
      return console.log("strack " + v + " [ticket/task text]\n\n  Add new ticket/task\n  For input multiline text, omit all parameters after add\n");
    case "config":
    case "cf":
      return console.log(("strack " + v + " [key [value]]\n\n  Work with config options\n  If key and value omited, show all config params\n") + "  If key is set, show key value\n  If key and value is set, write new value to config\n\nConfig options:\n" + '  user\t\t\tUser name\n  email\t\t\tUser email\n  log\t\t\tLog format, one of "tiny", "short", "long"\n  showDonedTasks\tShow done tasks' + ' when watch log, one of "true", "false"\n  verbose\t\tSet/unset verbose mode, one of "true", "false"\n');
    case "comment":
    case "c":
      return console.log("strack " + v + " id [comment]\n\n  Comment ticket/task\n  For input multiline comment, omit comment parameter\n");
    case "info":
    case "i":
      return console.log(("strack " + v + " id\n\n  Show detail information about ticket/task\n  ") + "Add . and shortcut symbol (0-z) to see only comments on ticket/task.\n");
    case "log":
    case "l":
      return console.log("strack " + v + " [pattern]\n\n  Show tracker log\n  If pattern is set, only tasks, that match this pattern will be displayed\n");
    case "remove":
    case "rm":
      return console.log("strack " + v + " id [id2, id3...]\n\n  Remove tickets/tasks from tracker\n");
    case "done":
    case "d":
      return console.log("strack " + v + " id new-state\n\n  Change state of ticket/task\n");
    case "states":
    case "st":
      return console.log(("strack " + v + " [group [new states]]\n\n  Work with project states.\n") + '  If optional params omited, show groups with states. Project have two groups\n ' + ' of states "initial" and "final". Final states are final in ticket/task processing\n' + '  If only group specified (it must be "initial" or "final"), will be shown params\n' + '  that belong to this group. New states REPLACE ALL group states.\n');
    case "fs":
      return console.log(("strack " + v + " [ext [keywords]]\n\n  Search keywords in file with ext ") + 'extension\n  Default ext is "js"\n  Default keywords is [config.defaultState]\n');
    case "touch":
    case "t":
      return console.log("strack " + v + " id\n\n  Change last access time for task with specified id.\n");
    default:
      return console.log(usage);
  }
};
exports.run = function() {
  var c, cId, cid, comment, commentList, config, data, ext, filename, filteredTags, format, id, ids, key, param, state, t, tags, ticket, tracker, value, word, _ref;
  config = new util.Config();
  tracker = new Tracker(config);
  tracker.load();
  if (tracker.created) {
    switch (process.argv[2]) {
      case "init":
        return "clean Tracker";
      case "add":
      case "a":
        if (3 < process.argv.length) {
          data = process.argv.slice(3);
          return tracker.addTicket(data.join(" "));
        } else {
          return util.readText(config.get("eof"), function(data) {
            return tracker.addTicket(data);
          });
        }
        break;
      case "edit":
      case "e":
        if (3 < process.argv.length) {
          id = process.argv[3];
          if ("." === id[0]) {
            id = "^" + id.substring(1);
            cid = process.argv[4];
            commentList = tracker.getComment(id, cid);
            if (commentList) {
              ticket = commentList[0], cId = commentList[1];
              c = ticket.comments[cId];
              if (tracker.canEditComment(id, cid)) {
                return util.editTextLines(c.comment, config.get("eof"), (function(text) {
                  if (text) {
                    return tracker.updateComment(id, cid, text);
                  }
                }), config.get("showLineNumbers"));
              } else {
                return console.log("You can't edit this comment");
              }
            } else {
              return console.log("Invalid comment id (" + cid + ")");
            }
          } else {
            t = tracker.getSingleTicket(id);
            return util.editTextLines(t.text, config.get("eof"), (function(text) {
              if (text) {
                t.text = text;
                return tracker.updateTicket(t);
              }
            }), config.get("showLineNumbers"));
          }
        } else {
          return console.log("To edit state text add id");
        }
        break;
      case "export":
        format = process.argv[3];
        if (format && (format === "txt" || format === "org" || format === "htm" || format === "html")) {
          filename = process.argv[4] || ("strack." + format);
          switch (format) {
            case "txt":
              return exp.toTxt(tracker, filename);
            case "org":
              return exp.toOrg(tracker, filename);
            case "htm":
            case "html":
              return exp.toHtml(tracker, filename);
          }
        } else {
          return console.log("Add format and filename");
        }
        break;
      case "config":
      case "cf":
        if (3 < process.argv.length) {
          key = process.argv[3];
        }
        if (key) {
          if (4 < process.argv.length) {
            value = process.argv[4];
          }
          if (value) {
            param = {};
            param[key] = value;
            return config.update(param);
          } else {
            return console.log("" + key + " = " + (config.get(key)));
          }
        } else {
          return config.dump();
        }
        break;
      case "log":
      case "l":
        if (3 < process.argv.length) {
          word = process.argv[3];
        }
        return tracker.log(word);
      case "done":
      case "d":
        if (4 < process.argv.length) {
          state = process.argv[3];
          id = process.argv[4];
          if (0 === id.indexOf(util.statePrefix)) {
            _ref = [id, state], state = _ref[0], id = _ref[1];
          }
          return tracker.changeState(id, state);
        } else {
          return console.log("To change state add id and new state");
        }
        break;
      case "states":
      case "st":
        if (4 < process.argv.length) {
          return tracker.updateStates(process.argv.slice(3));
        } else {
          return tracker.showStates(process.argv[3]);
        }
        break;
      case "info":
      case "i":
        if (3 < process.argv.length) {
          id = process.argv[3];
          if ("." === id[0]) {
            id = "^" + id.substring(1);
            return tracker.showComments(id);
          } else {
            return tracker.info(id);
          }
        } else {
          return console.log("Add id for ticket ");
        }
        break;
      case "remove":
      case "r":
      case "rm":
        if (3 < process.argv.length) {
          ids = process.argv.slice(3);
          return tracker.removeTickets(ids);
        }
        break;
      case "comment":
      case "c":
        if (3 < process.argv.length) {
          id = process.argv[3];
        }
        if (id) {
          if (4 < process.argv.length) {
            comment = process.argv.slice(4).join(" ");
          }
          if (!comment) {
            return util.readText(config.get("eof"), function(comment) {
              return tracker.commentTicket(id, comment);
            });
          } else {
            return tracker.commentTicket(id, comment);
          }
        } else {
          return console.log("Ticket id is missing");
        }
        break;
      case "help":
      case "h":
        return showHelp();
      case "fs":
        ext = "." + (3 < process.argv.length ? process.argv[3].toLowerCase() : "js");
        tags = 4 < process.argv.length ? process.argv.slice(4) : [config.get("defaultState")];
        filteredTags = [];
        tags.forEach(function(tag) {
          return filteredTags.push(tag.toLowerCase());
        });
        return util.listDir(process.cwd(), ext).forEach(function(file) {
          return parser.addTickets(file, tags, tracker, config);
        });
      case "touch":
      case "t":
        if (3 < process.argv.length) {
          t = tracker.getSingleTicket(process.argv[3]);
          t.modified = new Date();
          return tracker.updateTicket(t);
        } else {
          return console.log("Ticket id is missing");
        }
        break;
      default:
        return console.log(usage);
    }
  }
};