var Tracker, ccn, cfl, fs, path, sys, ucs, uct, util;
var __indexOf = Array.prototype.indexOf || function(item) {
  for (var i = 0, l = this.length; i < l; i++) {
    if (this[i] === item) return i;
  }
  return -1;
};
fs = require("fs");
util = require("./util");
ucs = util.colorizeString;
uct = util.colorizeText;
ccn = util.colorizeCommentNumber;
cfl = util.cutFirstLine;
sys = require("sys");
path = require("path");
Tracker = (function() {
  function Tracker(config) {
    this.config = config;
    this.editTimeLimit = 15 * 60000;
    this._create();
    this.created = true;
  }
  /*
  Create new object from params

  @param {Object} params Params object
  @api private
  */
  Tracker.prototype._create = function(params) {
    if (params == null) {
      params = {};
    }
    this.name = params.name || "";
    this.tickets = params.tickets || {};
    return this.states = params.states || {
      initial: ["todo", "bug", "accept"],
      final: ["done", "fixed", "closed"]
    };
  };
  /*
  Load tracker file. If file not exists, it will be created with
  default name in work dir

  @param {String} filename Path to tracker file
  @api public
  */
  Tracker.prototype.load = function() {
    var cwd, preWd, self, trackerFile;
    preWd = "";
    cwd = process.cwd();
    trackerFile = this.config.get("trackerFile");
    while (cwd !== preWd) {
      preWd = cwd;
      cwd = path.dirname(cwd);
      try {
        this.filename = "" + preWd + "/" + trackerFile;
        this._create(JSON.parse(fs.readFileSync(this.filename)));
        return;
      } catch (err) {

      }
    }
    cwd = process.cwd();
    this.filename = "" + cwd + "/" + trackerFile;
    this._create();
    if ("yes" === this.config.get("askBeforeCreate")) {
      this.created = false;
      self = this;
      return util.readOneLine("Create new tracker in " + cwd + " (yes/no) ? ", function(answer) {
        var _ref;
        if ((_ref = answer.toLowerCase()) === "yes" || _ref === "y") {
          return self.save();
        }
      });
    }
  };
  /*
  Save tracker file

  @param {String} filename Path to tracker file
  @api public
  */
  Tracker.prototype.save = function() {
    var data, k, name, states, tickStr, tickets, v, _ref;
    name = "\"name\": \"" + this.name + "\"";
    states = '"states":' + JSON.stringify(this.states);
    tickStr = '"tickets": {\n';
    tickets = [];
    _ref = this.tickets;
    for (k in _ref) {
      v = _ref[k];
      tickets.push('"' + k + '": ' + JSON.stringify(v));
    }
    tickStr += tickets.join(", \n") + "\n}";
    data = "{" + name + ",\n" + tickStr + ",\n" + states + "\n}\n";
    return fs.writeFileSync(this.filename, data);
  };
  /*
  Show project states

  @api public
  */
  Tracker.prototype.showStates = function(which) {
    if (which == null) {
      which = null;
    }
    if (null === which) {
      console.log("Project states:");
      console.log("Initial: " + (this.states.initial.join(', ')) + " ");
      return console.log("Final: " + (this.states.final.join(', ')) + " ");
    } else if (which === "initial" || which === "final") {
      return console.log("" + which + " states: " + (this.states[which].join(', ')));
    } else {
      return console.log('use "initial" or "final", not #{which}');
    }
  };
  /*
  Update states. Can update "initial" or "final" states

  @param {Array} params List of params. First param must be "initial" of "final",
                        rest params are values for initial or final states
  @api public
  */
  Tracker.prototype.updateStates = function(params) {
    var _ref;
    if ((_ref = params[0]) === "initial" || _ref === "final") {
      this.states[params[0]] = params.slice(1);
      this.save();
      if ("true" === this.config.get("verbose")) {
        return console.log("Project " + params[0] + " states updated to " + (params.slice(1).join(', ')));
      }
    } else {
      if ("true" === this.config.get("verbose")) {
        return console.log("Project states not updated");
      }
    }
  };
  /*
  Search ticket in tracker

  @param {String} ticketId Ticket id starting numbers
  @return {Array} result Tickets, which have id, strarting from ticketId
  @api private
  */
  Tracker.prototype._searchTicket = function(ticketId) {
    var id, result, t, tickets, _ref;
    result = [];
    if (ticketId.match(/^\^.$/)) {
      tickets = this._sortTickets();
      id = parseInt(ticketId[1], 36);
      if (id < tickets.length) {
        result.push(tickets[id]);
      }
    } else {
      _ref = this.tickets;
      for (id in _ref) {
        t = _ref[id];
        if (0 === id.indexOf(ticketId)) {
          result.push(t);
        }
      }
    }
    return result;
  };
  /*
  Get single ticket by id.
  If ticket id is not unique, method throws exception

  @param {String} ticketId Ticket id starting numbers
  @return {Object} ticket Ticket object
  @api public
  */
  Tracker.prototype.getSingleTicket = function(id) {
    var tickets;
    tickets = this._searchTicket(id, this.config);
    switch (tickets.length) {
      case 1:
        return tickets[0];
      case 0:
        console.log("Ticket with id, starting from '" + id + "' not found");
        break;
      default:
        console.log("Duplicate tickets with id = " + id + " ");
    }
    return process.exit(-1);
  };
  /*
  Add ticket to tracker

  @param {String} text Text of ticket
  @api public
  */
  Tracker.prototype.addTicket = function(text) {
    var d, meta, t;
    d = new Date();
    meta = util.parseText(text);
    t = {
      created: d,
      modified: d,
      author: this.config.makeUserDict(),
      text: text,
      id: util.createId(text, this.config),
      comments: [],
      log: []
    };
    this.tickets[t.id] = t;
    this.save();
    if ("true" === this.config.get("verbose")) {
      return console.log("You'we added ticket:\n" + text);
    }
  };
  /*
  Add unique ticket. Check ticket text for unique before adding

  @param {String} text Text of ticket
  @api public
  */
  Tracker.prototype.addUniqueTicket = function(text) {
    var id, t, _ref;
    _ref = this.tickets;
    for (id in _ref) {
      t = _ref[id];
      if (text.split("\n")[0] === t.text.split("\n")[0]) {
        if ("true" === this.config.get("verbose")) {
          console.log("Add duplicate ticket declined");
        }
        return;
      }
    }
    return this.addTicket(text);
  };
  /*
  Remove ticket from tracker.

  @param {String} id Ticket id  starting numbers
  @api public
  */
  Tracker.prototype.removeTicket = function(id) {
    var t;
    t = this.getSingleTicket(id);
    delete this.tickets[t.id];
    this.save();
    if ("true" === this.config.get("verbose")) {
      return console.log("Ticket with " + id.yellow + " removed");
    }
  };
  /*
  Remove tickets in list

  @param {Array} idList List of ids that must to deleted
  @api public
  */
  Tracker.prototype.removeTickets = function(idList) {
    var id, t, _i, _len;
    for (_i = 0, _len = idList.length; _i < _len; _i++) {
      id = idList[_i];
      try {
        t = this.getSingleTicket(id);
        delete this.tickets[t.id];
        if ("true" === this.config.get("verbose")) {
          console.log("Ticket with " + id.yellow + " removed");
        }
      } catch (err) {

      }
    }
    return this.save();
  };
  /*
  Change ticket text

  @param {String} ticketId Ticket id
  @param {Object} text New ticket text
  @api public
  */
  Tracker.prototype.changeTicket = function(id, text) {
    var t;
    t = this.getSingleTicket(id);
    t.author = this.config.makeUserDict();
    t.text = text;
    return this.updateTicket(t);
  };
  /*
  Update ticket

  @param {Object} ticket Ticket to update
  @api public
  */
  Tracker.prototype.updateTicket = function(ticket) {
    this.tickets[ticket.id] = ticket;
    return this.save();
  };
  /*
  Comment ticket

  @param {String} ticketId Ticket id
  @param {String} comment Comment text
  @api public
  */
  Tracker.prototype.commentTicket = function(id, text) {
    var comment, d, t;
    t = this.getSingleTicket(id);
    d = new Date();
    comment = {
      date: d,
      author: this.config.makeUserDict(),
      comment: text,
      id: util.createId(comment, this.config)
    };
    comment.sign = this._signComment(comment);
    t.comments.push(comment);
    this.updateTicket(t);
    if ("true" === this.config.get("verbose")) {
      return console.log("You add a comment:\n" + comment.comment);
    }
  };
  /*
  Sign comment

  @param {Object} comment Comment object
  @return {String} signature Comment signature
  @api private
  */
  Tracker.prototype._signComment = function(c) {
    return util.sign(c.comment + c.id, this.config);
  };
  /*
  Can edit this comment. Only author can edit his comment at a limited
  time (editTimeLimit)

  @param {String} ticketId Ticket id
  @param {String} commentId Comment id
  @return {Boolean} result REsult for current user (from config)
  @api public
  */
  Tracker.prototype.canEditComment = function(id, cid) {
    var c, cdate, commLst, d, ticket;
    commLst = this.getComment(id, cid);
    if (commLst) {
      ticket = commLst[0], id = commLst[1];
      c = ticket.comments[id];
      d = new Date();
      cdate = new Date(Date.parse(c.date));
      return this.editTimeLimit > d - cdate && c.sign === this._signComment(c);
    }
    return false;
  };
  /*
  Update comment on ticket

  @param {String} ticketId Ticket id
  @param {String} cid Commetn id
  @param {String} newComment New comment
  @api public
  */
  Tracker.prototype.updateComment = function(id, cid, newComment) {
    var c, cdate, comLst, d, i, sign, t;
    comLst = this.getComment(id, cid);
    if (comLst) {
      t = comLst[0], i = comLst[1];
      c = t.comments[i];
      d = new Date();
      cdate = new Date(Date.parse(c.date));
      if (this.editTimeLimit < d - cdate) {
        throw new Error("time limit to edit exeeded");
      } else {
        sign = this._signComment(c);
        if (sign === c.sign) {
          c.edited = d;
          c.comment = newComment;
          c.sign = this._signComment(c);
        } else {
          throw new Error("signature not match");
        }
      }
      this.updateTicket(t);
      if ("true" === this.config.get("verbose")) {
        return console.log("You update comment:\n" + newComment);
      }
    } else {
      if ("true" === this.config.get("verbose")) {
        return console.log("Error! can't update comment for ticket " + id);
      }
    }
  };
  /*
  Get comment by id

  @param {String} ticketId Ticket id
  @param {String} cid Comment id
  @return {Array|null} comment Return array of ticket and comment index in comments.
                               If comment not found, return null
  */
  Tracker.prototype.getComment = function(id, cid) {
    var c, com, i, t, _i, _len, _ref;
    t = this.getSingleTicket(id);
    c = false;
    i = 0;
    if ("." === cid[0] && t.comments[cid.substring(1)]) {
      return [t, cid.substring(1)];
    } else {
      _ref = t.comments;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        com = _ref[_i];
        if (0 === com.id.indexOf(cid)) {
          return [t, i];
        }
        i++;
      }
    }
    return null;
  };
  /*
  Show commnets on ticket

  @param {String} ticketId Ticket id
  @api public
  */
  Tracker.prototype.showComments = function(id) {
    var t, _ref;
    t = this.getSingleTicket(id);
    return this._showTicketComments(t, (_ref = util.getState(t.text, this.config), __indexOf.call(this.states.final, _ref) >= 0));
  };
  /*
  Change ticket state

  @param {String} ticketId Ticket id
  @param {String} newState New State value
  @api public
  */
  Tracker.prototype.changeState = function(id, newState) {
    var t, text;
    if (0 === newState.indexOf(util.statePrefix)) {
      t = this.getSingleTicket(id);
      if ("true" === this.config.get("verbose")) {
        console.log("State of: " + t.text + "\nchanged to " + newState);
      }
      text = util.replaceState(t.text, newState);
      t.text = text !== t.text ? text : "" + newState + " " + text;
      t.modified = new Date();
      return this.updateTicket(t);
    }
  };
  /*
  Show info on ticket

  @param {String} ticketId Ticket id
  @api public
  */
  Tracker.prototype.info = function(id) {
    return this._logOne(this.getSingleTicket(id), null);
  };
  /*
  Log one ticket full info

  @param {Object} ticket Ticket object
  @param {String} search Search string, default null
  @api private
  */
  Tracker.prototype._logOne = function(t, search) {
    var done, _ref;
    if (search == null) {
      search = null;
    }
    done = (_ref = util.getState(t.text, this.config), __indexOf.call(this.states.final, _ref) >= 0);
    console.log(ucs("Ticket: " + t.id.yellow, done, "grey", ""));
    console.log(ucs("Author: " + t.author.user + " <" + t.author.email + ">", done, "grey", ""));
    console.log(ucs("Created: " + t.created + "\n", done, "grey", ""));
    console.log(ucs("Last modified: " + t.modified, done, "grey", ""));
    console.log(uct(t.text, search, done));
    this._showTicketComments(t, done);
    return console.log("\n-----------------------------------------" + "---------------------------------------\n");
  };
  /*
  Show comments on ticket

  @param {Object} t Ticket object
  @param {Boolean} done Is ticket done
  */
  Tracker.prototype._showTicketComments = function(t, done) {
    var c, i, _i, _len, _ref, _results;
    if (0 < t.comments.length) {
      console.log(ucs("\nComments:\n", done, "grey", ""));
      i = 0;
      _ref = t.comments;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        c = _ref[_i];
        console.log(ccn(i, c.id) + ucs(" " + c.author.user + " <" + c.author.email + "> :", done, "grey", ""));
        console.log(ucs(c.comment, done, "grey", ""));
        _results.push(i++);
      }
      return _results;
    } else {
      return console.log("No comments");
    }
  };
  /*
  Sort tickets

  @return {Array} tickets Tickets, sorted by asc or desc, depends on config "sortOrder" option
  @api private
  */
  Tracker.prototype._sortTickets = function() {
    var id, neg, pos, states, t, tickets, _ref, _ref2, _ref3;
    tickets = [];
    _ref = this.tickets;
    for (id in _ref) {
      t = _ref[id];
      t.state = util.getState(t.text, this.config);
      tickets.push(t);
    }
    _ref2 = [1, -1], pos = _ref2[0], neg = _ref2[1];
    if ("desc" === this.config.get("sortOrder")) {
      _ref3 = [neg, pos], pos = _ref3[0], neg = _ref3[1];
    }
    states = this.states;
    tickets.sort(function(t1, t2) {
      var final1, final2, _ref, _ref2;
      final1 = (_ref = t1.state, __indexOf.call(states.final, _ref) >= 0);
      final2 = (_ref2 = t2.state, __indexOf.call(states.final, _ref2) >= 0);
      if (final1 === final2) {
        if (t1.modified === t2.modified) {
          return 0;
        } else if (t1.modified < t2.modified) {
          return pos;
        } else {
          return neg;
        }
      } else if (final1) {
        return pos;
      } else {
        return neg;
      }
    });
    return tickets;
  };
  /*
  Log + search tickets

  @param {String} search Search string, default null
  @api public
  */
  Tracker.prototype.log = function(search) {
    var comments, done, i, id, num, stat, state, t, total, _i, _len, _ref;
    if (search == null) {
      search = null;
    }
    stat = {
      todo: 0,
      done: null === search ? 0 : void 0
    };
    i = -1;
    _ref = this._sortTickets();
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      t = _ref[_i];
      i++;
      state = util.getState(t.text, this.config);
      done = __indexOf.call(this.states.final, state) >= 0;
      if (stat) {
        if (done) {
          stat.done++;
          if ("false" === this.config.get("showDone")) {
            continue;
          }
        } else {
          stat.todo++;
        }
      }
      if (null === search || 0 <= t.text.indexOf(search)) {
        num = 36 > i ? ucs(" ^" + (i.toString(36)) + " ", done, "grey", "") : "    ";
        if ("yes" === this.config.get("useZebra") && 0 === i % 2) {
          num = num.green.inverse.bold;
        }
        comments = 0 < t.comments.length ? ucs(" [c:" + t.comments.length + "]\t", done, "grey", "") : "     \t";
        switch (this.config.get("log")) {
          case "tiny":
            console.log(("" + num + "\t" + (uct(cfl(t.text, 60), null, done)) + " ") + ("\t" + comments).blue);
            break;
          case "long":
            this._logOne(t, search);
            break;
          default:
            id = cfl(t.id, 10).yellow;
            if ("yes" === this.config.get("useZebra") && 0 === i % 2) {
              id = id.inverse.bold;
            }
            console.log(("" + id + num + "\t" + (uct(cfl(t.text, 60), search, done)) + "\t" + comments) + ucs("" + (util.formatDateTime(t.modified)) + "\t" + t.author.user, done, "grey", ""));
        }
      }
    }
    if (null === search) {
      total = stat.todo + stat.done;
      if (0 < total) {
        return console.log("Tickets: " + stat.done + "/" + (stat.todo + stat.done));
      } else {
        return console.log("No tickets yet");
      }
    }
  };
  return Tracker;
})();
exports.Tracker = Tracker;